@@ -21,6 +21,10 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <sys/file.h>
+
+#include <unistd.h>
+#include <stdlib.h>
+
 #include <stdio.h>
 
 main()
