@@ -212,7 +212,7 @@ parse_command ()
     {
       command_to_execute = get_string_value ("PROMPT_COMMAND");
       if (command_to_execute)
-	execute_prompt_command (command_to_execute);
+	execute_variable_command (command_to_execute, "PROMPT_COMMAND");
 
       if (running_under_emacs == 2)
 	send_pwd_to_eterm ();	/* Yuck */
