@@ -14,13 +14,29 @@
 #	make
 #	CC="cc -x2.3" ./link.sh
 
+# Copyright (C) 1989-2002 Free Software Foundation, Inc.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2, or (at your option)
+# any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+
 set -x
 
 rm -f bash
 
 if [ -z "$CC" ]
 then
-	if [ -f /unix -a ! -f /xenix ]
+	if [ -f /unix ] && [ ! -f /xenix ]
 	then
 		CC="cc -xenix"
 	else
